@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009-2011 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2012 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -28,14 +28,11 @@
 #include <math.h>
 #include <gusb.h>
 #include <libsoup/soup.h>
+#include <colorhug.h>
 
-#include "ch-common.h"
 #include "ch-markdown.h"
 #include "ch-flash-md.h"
 
-/* don't change this unless you want to provide firmware updates */
-#define COLORHUG_FIRMWARE_LOCATION	"http://www.hughski.com/downloads/colorhug/firmware/"
-
 typedef struct {
 	gchar		*filename;
 	gchar		*checksum;
@@ -53,15 +50,11 @@ typedef struct {
 	GUsbDevice	*device;
 	GUsbDeviceList	*device_list;
 	SoupSession	*session;
-	guint		 flash_idx;
-	gsize		 flash_chunk_len;
-	guint8		 flash_buffer[64];
 	ChMarkdown	*markdown;
+	ChDeviceQueue	*device_queue;
+	GSettings	*settings;
 } ChFlashPrivate;
 
-static void	 ch_flash_read_firmware_chunk	(ChFlashPrivate *priv);
-static void	 ch_flash_write_firmware_chunk	(ChFlashPrivate *priv);
-
 /**
  * ch_flash_error_dialog:
  **/
@@ -188,175 +181,6 @@ ch_flash_close_button_cb (GtkWidget *widget, ChFlashPrivate *priv)
 	gtk_widget_destroy (widget);
 }
 
-
-/**
- * ch_flash_calculate_checksum:
- **/
-static guint8
-ch_flash_calculate_checksum (guint8 *data,
-			     gsize len)
-{
-	guint8 checksum = 0xff;
-	guint i;
-	for (i = 0; i < len; i++)
-		checksum ^= data[i];
-	return checksum;
-}
-
-/**
- * ch_flash_write_flash_cb:
- **/
-static void
-ch_flash_write_flash_cb (GObject *source,
-			 GAsyncResult *res,
-			 gpointer user_data)
-{
-	const gchar *title;
-	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
-	gboolean ret;
-	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
-
-	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
-	if (!ret) {
-		ch_flash_error_do_not_panic (priv);
-		/* TRANSLATORS: the flash write failed */
-		title = _("Failed to write the flash");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* write the next chunk */
-	ch_flash_write_firmware_chunk (priv);
-out:
-	return;
-}
-
-/**
- * ch_flash_write_flash:
- **/
-static void
-ch_flash_write_flash (ChFlashPrivate *priv,
-		      guint16 address,
-		      guint8 *data,
-		      gsize len)
-{
-	guint16 addr_le;
-	guint8 buffer_tx[64];
-
-	/* set address, length, checksum, data */
-	addr_le = GUINT16_TO_LE (address);
-	memcpy (buffer_tx + 0, &addr_le, 2);
-	buffer_tx[2] = len;
-	buffer_tx[3] = ch_flash_calculate_checksum (data, len);
-	memcpy (buffer_tx + 4, data, len);
-
-	/* hit hardware */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_WRITE_FLASH,
-				       buffer_tx,
-				       len + 4,
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
-				       ch_flash_write_flash_cb,
-				       priv);
-}
-
-/**
- * ch_flash_read_flash_cb:
- **/
-static void
-ch_flash_read_flash_cb (GObject *source,
-			GAsyncResult *res,
-			gpointer user_data)
-{
-	const gchar *title;
-	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
-	gboolean ret;
-	gchar *str = NULL;
-	GError *error = NULL;
-	guint8 expected_checksum;
-	GUsbDevice *device = G_USB_DEVICE (source);
-
-	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
-	if (!ret) {
-		ch_flash_error_do_not_panic (priv);
-		/* TRANSLATORS: the flash failed and the checksum is wrong */
-		title = _("Failed to verify the flash");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* verify checksum */
-	expected_checksum = ch_flash_calculate_checksum (priv->flash_buffer + 1,
-							 priv->flash_chunk_len);
-	if (priv->flash_buffer[0] != expected_checksum) {
-		/* TRANSLATORS: the write failed */
-		str = g_strdup_printf (_("Checksum failed at 0x%04x"),
-				       CH_EEPROM_ADDR_RUNCODE + priv->flash_idx);
-		ch_flash_error_do_not_panic (priv);
-		ch_flash_error_dialog (priv,
-				       _("Failed to verify the checksum"),
-				       str);
-		goto out;
-	}
-
-	/* failed to verify chunk */
-	if (memcmp (priv->firmware_data + priv->flash_idx,
-		    priv->flash_buffer + 1,
-		    priv->flash_chunk_len) != 0) {
-		/* TRANSLATORS: the write failed */
-		str = g_strdup_printf (_("Verification failed at 0x%04x (len %i)"),
-				       CH_EEPROM_ADDR_RUNCODE + priv->flash_idx,
-				       (guint) priv->flash_chunk_len);
-		ch_flash_error_do_not_panic (priv);
-		/* TRANSLATORS: the write failed */
-		title = _("Failed to verify the flash");
-		ch_flash_error_dialog (priv, title, str);
-		goto out;
-	}
-
-	/* read the next chunk */
-	priv->flash_idx += priv->flash_chunk_len;
-	ch_flash_read_firmware_chunk (priv);
-out:
-	g_free (str);
-	return;
-}
-
-/**
- * ch_flash_read_flash:
- **/
-static void
-ch_flash_read_flash (ChFlashPrivate *priv,
-		     guint16 address,
-		     gsize len)
-{
-	guint8 buffer_tx[3];
-	guint16 addr_le;
-
-	/* set address, length, checksum, data */
-	addr_le = GUINT16_TO_LE (address);
-	memcpy (buffer_tx + 0, &addr_le, 2);
-	buffer_tx[2] = len;
-
-	/* hit hardware */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_READ_FLASH,
-				       buffer_tx,
-				       sizeof(buffer_tx),
-				       priv->flash_buffer,
-				       len + 1,
-				       NULL, /* cancellable */
-				       ch_flash_read_flash_cb,
-				       priv);
-}
-
 /**
  * ch_flash_set_flash_success_1_cb:
  **/
@@ -370,10 +194,10 @@ ch_flash_set_flash_success_1_cb (GObject *source,
 	gboolean ret;
 	GError *error = NULL;
 	GtkWidget *widget;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		ch_flash_error_do_not_panic (priv);
 		/* TRANSLATORS: we can only set the flash success
@@ -405,7 +229,6 @@ ch_flash_set_flash_success_1 (ChFlashPrivate *priv)
 {
 	const gchar *title;
 	GtkWidget *widget;
-	guint8 flash_success = 0x01;
 
 	/* update UI */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_status"));
@@ -414,13 +237,12 @@ ch_flash_set_flash_success_1 (ChFlashPrivate *priv)
 	gtk_label_set_label (GTK_LABEL (widget), title);
 
 	/* need to boot into bootloader */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_SET_FLASH_SUCCESS,
-				       &flash_success,
-				       1,
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
+	ch_device_queue_set_flash_success (priv->device_queue,
+					   priv->device,
+					   0x01);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
 				       ch_flash_set_flash_success_1_cb,
 				       priv);
 }
@@ -462,10 +284,10 @@ ch_flash_boot_flash_cb (GObject *source,
 	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	gboolean ret;
 	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		ch_flash_error_do_not_panic (priv);
 		/* TRANSLATORS: the new firmware will not load */
@@ -483,32 +305,29 @@ out:
 }
 
 /**
- * ch_flash_read_firmware_chunk:
+ * ch_flash_verify_firmware_cb:
  **/
 static void
-ch_flash_read_firmware_chunk (ChFlashPrivate *priv)
+ch_flash_verify_firmware_cb (GObject *source,
+			     GAsyncResult *res,
+			     gpointer user_data)
 {
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
+	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	const gchar *title;
+	gboolean ret;
+	GError *error = NULL;
 	GtkWidget *widget;
-	gfloat complete;
 
-	/* work out percentage complete */
-	complete = (gfloat) priv->flash_idx / (gfloat) priv->firmware_len;
-
-	/* update UI */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "progressbar_status"));
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), complete);
-
-	/* any more data to read? */
-	if (priv->flash_idx < priv->firmware_len) {
-		if (priv->flash_idx + priv->flash_chunk_len > priv->firmware_len)
-			priv->flash_chunk_len = priv->firmware_len - priv->flash_idx;
-		g_debug ("Reading at %04x size %" G_GSIZE_FORMAT,
-			 CH_EEPROM_ADDR_RUNCODE + priv->flash_idx,
-			 priv->flash_chunk_len);
-		ch_flash_read_flash (priv,
-				     CH_EEPROM_ADDR_RUNCODE + priv->flash_idx,
-				     priv->flash_chunk_len);
+	/* get data */
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
+	if (!ret) {
+		ch_flash_error_do_not_panic (priv);
+		/* TRANSLATORS: tell the device the firmware is no
+		 * longer known working */
+		title = _("Failed to verify the firmware");
+		ch_flash_error_dialog (priv, title, error->message);
+		g_error_free (error);
 		goto out;
 	}
 
@@ -534,13 +353,11 @@ ch_flash_read_firmware_chunk (ChFlashPrivate *priv)
 	priv->planned_replug = TRUE;
 
 	/* boot into new code */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_BOOT_FLASH,
-				       NULL, /* buffer_in */
-				       0, /* buffer_in_len */
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
+	ch_device_queue_boot_flash (priv->device_queue,
+				    priv->device);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONFATAL_ERRORS,
+				       NULL,
 				       ch_flash_boot_flash_cb,
 				       priv);
 out:
@@ -548,36 +365,31 @@ out:
 }
 
 /**
- * ch_flash_write_firmware_chunk:
+ * ch_flash_write_firmware_cb:
  **/
 static void
-ch_flash_write_firmware_chunk (ChFlashPrivate *priv)
+ch_flash_write_firmware_cb (GObject *source,
+			    GAsyncResult *res,
+			    gpointer user_data)
 {
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
+	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	const gchar *title;
+	gboolean ret;
+	GError *error = NULL;
 	GtkWidget *widget;
-	gfloat complete;
 
-	/* work out percentage complete */
-	complete = (gfloat) priv->flash_idx / (gfloat) priv->firmware_len;
-
-	/* update UI */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "progressbar_status"));
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), complete);
-
-	/* write data */
-	if (priv->flash_idx < priv->firmware_len) {
-		if (priv->flash_idx + priv->flash_chunk_len > priv->firmware_len)
-			priv->flash_chunk_len = priv->firmware_len - priv->flash_idx;
-		g_debug ("Writing at %04x size %" G_GSIZE_FORMAT,
-			 CH_EEPROM_ADDR_RUNCODE + priv->flash_idx,
-			 priv->flash_chunk_len);
-		ch_flash_write_flash (priv,
-				      CH_EEPROM_ADDR_RUNCODE + priv->flash_idx,
-				      (guint8 *) priv->firmware_data + priv->flash_idx,
-				      priv->flash_chunk_len);
-		priv->flash_idx += priv->flash_chunk_len;
+	/* get data */
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
+	if (!ret) {
+		ch_flash_error_do_not_panic (priv);
+		/* TRANSLATORS: tell the device the firmware is no
+		 * longer known working */
+		title = _("Failed to write the firmware");
+		ch_flash_error_dialog (priv, title, error->message);
+		g_error_free (error);
 		goto out;
-	};
+	}
 
 	/* update UI */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_status"));
@@ -585,101 +397,16 @@ ch_flash_write_firmware_chunk (ChFlashPrivate *priv)
 	 * verify it before we tell the device it was successfull */
 	title = _("Verifying new firmware...");
 	gtk_label_set_label (GTK_LABEL (widget), title);
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "progressbar_status"));
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), 0.0f);
 
-	/* verify by reading in 60 byte chunks */
-	priv->flash_idx = 0;
-	priv->flash_chunk_len = 60;
-	ch_flash_read_firmware_chunk (priv);
-out:
-	return;
-}
-
-/**
- * ch_flash_erase_flash_cb:
- **/
-static void
-ch_flash_erase_flash_cb (GObject *source,
-			 GAsyncResult *res,
-			 gpointer user_data)
-{
-	const gchar *title;
-	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
-	gboolean ret;
-	GError *error = NULL;
-	GtkWidget *widget;
-	GUsbDevice *device = G_USB_DEVICE (source);
-
-	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
-	if (!ret) {
-		ch_flash_error_do_not_panic (priv);
-		/* TRANSLATORS: clear enough flash for the new firmware */
-		title = _("Failed to erase the flash");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* update UI */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_status"));
-	/* TRANSLATORS: now write the new firmware chunks */
-	title = _("Writing new firmware...");
-	gtk_label_set_label (GTK_LABEL (widget), title);
-
-	/* write in 32 byte chunks */
-	priv->flash_idx = 0;
-	priv->flash_chunk_len = CH_FLASH_TRANSFER_BLOCK_SIZE;
-	ch_flash_write_firmware_chunk (priv);
-out:
-	return;
-}
-
-/**
- * ch_flash_set_flash_success_0_cb:
- **/
-static void
-ch_flash_set_flash_success_0_cb (GObject *source,
-				 GAsyncResult *res,
-				 gpointer user_data)
-{
-	const gchar *title;
-	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
-	gboolean ret;
-	GError *error = NULL;
-	guint16 addr_le;
-	guint8 buffer_tx[4];
-	guint16 len_le;
-	GUsbDevice *device = G_USB_DEVICE (source);
-
-	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
-	if (!ret) {
-		ch_flash_error_do_not_panic (priv);
-		/* TRANSLATORS: tell the device the firmware is no
-		 * longer known working */
-		title = _("Failed to set the flash success false");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* set address, length */
-	addr_le = GUINT16_TO_LE (CH_EEPROM_ADDR_RUNCODE);
-	memcpy (buffer_tx + 0, &addr_le, 2);
-	len_le = GUINT16_TO_LE (priv->firmware_len);
-	memcpy (buffer_tx + 2, &len_le, 2);
-
-	/* erase enough flash for the program code */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_ERASE_FLASH,
-				       buffer_tx,
-				       sizeof(buffer_tx),
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
-				       ch_flash_erase_flash_cb,
+	/* verify firmware */
+	ch_device_queue_verify_firmware (priv->device_queue,
+					 priv->device,
+					 priv->firmware_data,
+					 priv->firmware_len);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       ch_flash_verify_firmware_cb,
 				       priv);
 out:
 	return;
@@ -693,23 +420,25 @@ ch_flash_set_flash_success_0 (ChFlashPrivate *priv)
 {
 	const gchar *title;
 	GtkWidget *widget;
-	guint8 flash_success = 0x00;
 
 	/* update UI */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_status"));
-	/* TRANSLATORS: tell the device the firmware is no longer known working */
-	title = _("Clearing flash success...");
+	/* TRANSLATORS: now write the new firmware chunks */
+	title = _("Writing new firmware...");
 	gtk_label_set_label (GTK_LABEL (widget), title);
 
-	/* need to boot into bootloader */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_SET_FLASH_SUCCESS,
-				       &flash_success,
-				       1,
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
-				       ch_flash_set_flash_success_0_cb,
+	/* write firmware */
+	ch_device_queue_set_flash_success (priv->device_queue,
+					   priv->device,
+					   0x00);
+	ch_device_queue_write_firmware (priv->device_queue,
+					priv->device,
+					priv->firmware_data,
+					priv->firmware_len);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       ch_flash_write_firmware_cb,
 				       priv);
 }
 
@@ -749,10 +478,10 @@ ch_flash_reset_cb (GObject *source,
 	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	gboolean ret;
 	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		ch_flash_error_do_not_panic (priv);
 		/* TRANSLATORS: we restart the device in bootloader mode */
@@ -794,13 +523,11 @@ ch_flash_got_firmware_data (ChFlashPrivate *priv)
 	priv->planned_replug = TRUE;
 
 	/* need to boot into bootloader */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_RESET,
-				       NULL, /* buffer_in */
-				       0, /* buffer_in_len */
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
+	ch_device_queue_reset (priv->device_queue,
+			       priv->device);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONFATAL_ERRORS,
+				       NULL,
 				       ch_flash_reset_cb,
 				       priv);
 }
@@ -960,6 +687,7 @@ ch_flash_flash_button_cb (GtkWidget *widget, ChFlashPrivate *priv)
 	SoupURI *base_uri = NULL;
 	SoupMessage *msg = NULL;
 	gboolean ret;
+	gchar *server_uri = NULL;
 	gchar *uri = NULL;
 
 	/* show the user any warning dialog */
@@ -993,7 +721,8 @@ ch_flash_flash_button_cb (GtkWidget *widget, ChFlashPrivate *priv)
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), 0.0f);
 
 	/* download file */
-	uri = g_build_filename (COLORHUG_FIRMWARE_LOCATION,
+	server_uri = g_settings_get_string (priv->settings, "firmware-uri");
+	uri = g_build_filename (server_uri,
 				priv->filename,
 				NULL);
 	g_debug ("Downloading %s", uri);
@@ -1017,6 +746,7 @@ ch_flash_flash_button_cb (GtkWidget *widget, ChFlashPrivate *priv)
 	soup_session_queue_message (priv->session, msg,
 				    ch_flash_got_firmware_cb, priv);
 out:
+	g_free (server_uri);
 	g_free (uri);
 	if (base_uri != NULL)
 		soup_uri_free (base_uri);
@@ -1114,6 +844,7 @@ ch_flash_got_metadata_cb (SoupSession *session,
 	GError *error = NULL;
 	GPtrArray *updates = NULL;
 	guint i;
+	gboolean enable_test_firmware;
 
 	/* we failed */
 	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
@@ -1133,6 +864,10 @@ ch_flash_got_metadata_cb (SoupSession *session,
 		goto out;
 	}
 
+	/* this is a session configurable */
+	enable_test_firmware = g_settings_get_boolean (priv->settings,
+						       "enable-test-firmware");
+
 	/* parse file */
 	updates = ch_flash_md_parse_data (msg->response_body->data,
 					  &error);
@@ -1150,11 +885,29 @@ ch_flash_got_metadata_cb (SoupSession *session,
 		if (!ch_flash_version_is_newer (priv, update->version))
 			break;
 
+		/* is the state correct? */
+		if (update->state != CH_FLASH_MD_STATE_STABLE && !enable_test_firmware)
+			continue;
+
 		/* add info text */
-		g_string_append_printf (priv->update_details,
-					"%s\n", update->info->str);
-		g_string_append_printf (priv->warning_details,
-					"%s\n", update->warning->str);
+		if (update->info->len > 0) {
+			g_string_append_printf (priv->update_details,
+						"%s\n", update->info->str);
+		}
+
+		/* add warning text */
+		if (update->state == CH_FLASH_MD_STATE_TESTING) {
+			g_string_append_printf (priv->warning_details,
+						"**%s**\n%s\n",
+						/* TRANSLATORS: this is test firmware */
+						_("This is a test firmware not intended for general release."),
+						/* TRANSLATORS: all bets are off */
+						_("This firmware has not been widely tested and may not work as expected."));
+		}
+		if (update->warning->len > 0) {
+			g_string_append_printf (priv->warning_details,
+						"%s\n", update->warning->str);
+		}
 
 		/* save newest available firmware */
 		if (priv->filename == NULL) {
@@ -1200,10 +953,10 @@ ch_flash_set_lost_or_stolen_cb (GObject *source,
 	GError *error = NULL;
 	GString *msg = NULL;
 	GtkWidget *widget;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		ch_flash_error_do_not_panic (priv);
 		/* TRANSLATORS: the device refused our request to mark
@@ -1258,16 +1011,13 @@ out:
 static void
 ch_flash_set_lost_or_stolen (ChFlashPrivate *priv)
 {
-	guint8 flash_success = 0xff;
-
 	/* TODO: need to erase device as well */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_SET_FLASH_SUCCESS,
-				       &flash_success,
-				       1,
-				       NULL, /* buffer_out */
-				       0, /* buffer_out_len */
-				       NULL, /* cancellable */
+	ch_device_queue_set_flash_success (priv->device_queue,
+					   priv->device,
+					   0xff);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
 				       ch_flash_set_lost_or_stolen_cb,
 				       priv);
 }
@@ -1284,6 +1034,7 @@ ch_flash_got_blacklist_cb (SoupSession *session,
 	const gchar *title;
 	gboolean ret;
 	gchar **lines = NULL;
+	gchar *server_uri = NULL;
 	gchar *uri = NULL;
 	GError *error = NULL;
 	GtkWidget *widget;
@@ -1340,7 +1091,8 @@ ch_flash_got_blacklist_cb (SoupSession *session,
 	}
 
 	/* get the latest manifest file */
-	uri = g_build_filename (COLORHUG_FIRMWARE_LOCATION,
+	server_uri = g_settings_get_string (priv->settings, "firmware-uri");
+	uri = g_build_filename (server_uri,
 				"metadata.xml",
 				NULL);
 	base_uri = soup_uri_new (uri);
@@ -1360,6 +1112,7 @@ ch_flash_got_blacklist_cb (SoupSession *session,
 out:
 	if (base_uri != NULL)
 		soup_uri_free (base_uri);
+	g_free (server_uri);
 	g_free (uri);
 	g_strfreev (lines);
 }
@@ -1371,6 +1124,7 @@ static void
 ch_flash_got_device_data (ChFlashPrivate *priv)
 {
 	const gchar *title;
+	gchar *server_uri = NULL;
 	gchar *str = NULL;
 	gchar *uri = NULL;
 	gchar *user_agent = NULL;
@@ -1448,7 +1202,8 @@ ch_flash_got_device_data (ChFlashPrivate *priv)
 	gtk_widget_show (widget);
 
 	/* get the latest blacklist file */
-	uri = g_build_filename (COLORHUG_FIRMWARE_LOCATION,
+	server_uri = g_settings_get_string (priv->settings, "firmware-uri");
+	uri = g_build_filename (server_uri,
 				"BLACKLIST",
 				NULL);
 	base_uri = soup_uri_new (uri);
@@ -1472,6 +1227,7 @@ out:
 		soup_uri_free (base_uri);
 	g_free (user_agent);
 	g_free (str);
+	g_free (server_uri);
 	g_free (uri);
 }
 
@@ -1487,10 +1243,10 @@ ch_flash_get_serial_number_cb (GObject *source,
 	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	gboolean ret;
 	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		/* TRANSLATORS: the request failed */
 		title = _("Failed to contact ColorHug");
@@ -1517,10 +1273,10 @@ ch_flash_get_firmware_version_cb (GObject *source,
 	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
 	gboolean ret;
 	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
+	ChDeviceQueue *device_queue = CH_DEVICE_QUEUE (source);
 
 	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
+	ret = ch_device_queue_process_finish (device_queue, res, &error);
 	if (!ret) {
 		/* TRANSLATORS: the request failed */
 		title = _("Failed to contact ColorHug");
@@ -1536,57 +1292,18 @@ ch_flash_get_firmware_version_cb (GObject *source,
 	}
 
 	/* get the serial number */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_GET_SERIAL_NUMBER,
-				       NULL, /* buffer_in */
-				       0, /* buffer_in_len */
-				       (guint8 *) &priv->serial_number,
-				       4,
-				       NULL, /* cancellable */
+	ch_device_queue_get_serial_number (priv->device_queue,
+					   priv->device,
+					   &priv->serial_number);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
 				       ch_flash_get_serial_number_cb,
 				       priv);
 out:
 	return;
 }
 
-/**
- * ch_flash_get_hardware_version_cb:
- **/
-static void
-ch_flash_get_hardware_version_cb (GObject *source,
-				  GAsyncResult *res,
-				  gpointer user_data)
-{
-	const gchar *title;
-	ChFlashPrivate *priv = (ChFlashPrivate *) user_data;
-	gboolean ret;
-	GError *error = NULL;
-	GUsbDevice *device = G_USB_DEVICE (source);
-
-	/* get data */
-	ret = ch_device_write_command_finish (device, res, &error);
-	if (!ret) {
-		/* TRANSLATORS: the request failed */
-		title = _("Failed to contact ColorHug");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* get the firmware version */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_GET_FIRMWARE_VERSION,
-				       NULL, /* buffer_in */
-				       0, /* buffer_in_len */
-				       (guint8 *) priv->firmware_version,
-				       6,
-				       NULL, /* cancellable */
-				       ch_flash_get_firmware_version_cb,
-				       priv);
-out:
-	return;
-}
-
 /**
  * ch_flash_get_fake_device:
  **/
@@ -1622,7 +1339,7 @@ ch_flash_got_device (ChFlashPrivate *priv)
 		goto fake_device;
 
 	/* open device */
-	ret = g_usb_device_open (priv->device, &error);
+	ret = ch_device_open (priv->device, &error);
 	if (!ret) {
 		/* TRANSLATORS: permissions error perhaps? */
 		title = _("Failed to open device");
@@ -1630,27 +1347,6 @@ ch_flash_got_device (ChFlashPrivate *priv)
 		g_error_free (error);
 		return;
 	}
-	ret = g_usb_device_set_configuration (priv->device,
-					      CH_USB_CONFIG,
-					      &error);
-	if (!ret) {
-		/* TRANSLATORS: we can't set the device config */
-		title = _("Failed to set configuration");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		return;
-	}
-	ret = g_usb_device_claim_interface (priv->device,
-					    CH_USB_INTERFACE,
-					    G_USB_DEVICE_CLAIM_INTERFACE_BIND_KERNEL_DRIVER,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: we can't claim the interface for our sole usage */
-		title = _("Failed to claim interface");
-		ch_flash_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		return;
-	}
 
 fake_device:
 	/* initial detection */
@@ -1665,15 +1361,19 @@ fake_device:
 		gtk_label_set_label (GTK_LABEL (widget), title);
 	}
 
-	/* get the firmware version */
-	ch_device_write_command_async (priv->device,
-				       CH_CMD_GET_HARDWARE_VERSION,
-				       NULL, /* buffer_in */
-				       0, /* buffer_in_len */
-				       &priv->hardware_version,
-				       1,
-				       NULL, /* cancellable */
-				       ch_flash_get_hardware_version_cb,
+	/* get the hardware and firmware version */
+	ch_device_queue_get_hardware_version (priv->device_queue,
+					      priv->device,
+					      &priv->hardware_version);
+	ch_device_queue_get_firmware_ver (priv->device_queue,
+					  priv->device,
+					  &priv->firmware_version[0],
+					  &priv->firmware_version[1],
+					  &priv->firmware_version[2]);
+	ch_device_queue_process_async (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       ch_flash_get_firmware_version_cb,
 				       priv);
 }
 
@@ -1885,6 +1585,17 @@ ch_flash_ignore_cb (const gchar *log_domain, GLogLevelFlags log_level,
 {
 }
 
+static void
+ch_flash_device_queue_progress_changed_cb (ChDeviceQueue *device_queue,
+					   guint percentage,
+					   ChFlashPrivate *priv)
+{
+	GtkWidget *widget;
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "progressbar_status"));
+	g_debug ("queue complete %i%%", percentage);
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget), (gdouble) percentage / 100.0f);
+}
+
 /**
  * main:
  **/
@@ -1930,10 +1641,16 @@ main (int argc, char **argv)
 	g_option_context_free (context);
 
 	priv = g_new0 (ChFlashPrivate, 1);
+	priv->settings = g_settings_new ("com.hughski.colorhug-client");
 	priv->filename = filename;
 	priv->update_details = g_string_new ("");
 	priv->warning_details = g_string_new ("");
 	priv->markdown = ch_markdown_new ();
+	priv->device_queue = ch_device_queue_new ();
+	g_signal_connect (priv->device_queue,
+			  "progress-changed",
+			  G_CALLBACK (ch_flash_device_queue_progress_changed_cb),
+			  priv);
 	priv->usb_ctx = g_usb_context_new (NULL);
 	priv->device_list = g_usb_device_list_new (priv->usb_ctx);
 	g_signal_connect (priv->device_list, "device-added",
@@ -1966,6 +1683,8 @@ main (int argc, char **argv)
 		g_string_free (priv->warning_details, TRUE);
 	if (priv->device_list != NULL)
 		g_object_unref (priv->device_list);
+	if (priv->device_queue != NULL)
+		g_object_unref (priv->device_queue);
 	if (priv->usb_ctx != NULL)
 		g_object_unref (priv->usb_ctx);
 	if (priv->builder != NULL)
@@ -1974,6 +1693,8 @@ main (int argc, char **argv)
 		g_object_unref (priv->session);
 	if (priv->markdown != NULL)
 		g_object_unref (priv->markdown);
+	if (priv->settings != NULL)
+		g_object_unref (priv->settings);
 	g_free (priv->filename);
 	g_free (priv->checksum);
 	g_free (priv->firmware_data);
