@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009-2011 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2012 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -26,15 +26,15 @@
 #include <locale.h>
 #include <colord.h>
 #include <math.h>
-
-#include "ch-client.h"
+#include <gusb.h>
+#include <colorhug.h>
 
 typedef struct {
 	GtkBuilder	*builder;
 	GtkApplication	*application;
-	ChClient	*client;
 	guint8		 leds_old;
 	CdColorRGB	 value_max;
+	ChDeviceQueue	*device_queue;
 	GUsbDevice	*device;
 } ChUtilPrivate;
 
@@ -82,45 +82,18 @@ ch_util_set_default_calibration (ChUtilPrivate *priv)
 
 	/* set to HW */
 	cd_mat33_set_identity (&calibration);
-	ret = ch_device_cmd_set_calibration (priv->device,
-					     0,
-					     &calibration,
-					     CH_CALIBRATION_TYPE_ALL,
-					     "Default unity value",
-					     &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to set calibration");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* set to HW */
-	ret = ch_device_cmd_set_post_scale (priv->device,
-					    post_scale,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: post scale is applied after the XYZ conversion */
-		title = _("Failed to set post scale");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* set to HW */
-	ret = ch_device_cmd_set_pre_scale (priv->device,
-					   pre_scale,
-					   &error);
-	if (!ret) {
-		/* TRANSLATORS: pre scale is applied after the dRGB
-		 * sample but before the XYZ conversion */
-		title = _("Failed to set pre scale");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
+	ch_device_queue_set_calibration (priv->device_queue,
+					 priv->device,
+					 0,
+					 &calibration,
+					 CH_CALIBRATION_TYPE_ALL,
+					 "Default unity value");
+	ch_device_queue_set_post_scale (priv->device_queue,
+					priv->device,
+					post_scale);
+	ch_device_queue_set_pre_scale (priv->device_queue,
+				       priv->device,
+				       pre_scale);
 	/* set to HW */
 	calibration_map[0] = 0x00;
 	calibration_map[1] = 0x00;
@@ -128,9 +101,13 @@ ch_util_set_default_calibration (ChUtilPrivate *priv)
 	calibration_map[3] = 0x00;
 	calibration_map[4] = 0x00;
 	calibration_map[5] = 0x00;
-	ret = ch_device_cmd_set_calibration_map (priv->device,
-						 calibration_map,
-						 &error);
+	ch_device_queue_set_calibration_map (priv->device_queue,
+					     priv->device,
+					     calibration_map);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set the calibration map");
@@ -168,13 +145,43 @@ ch_util_refresh (ChUtilPrivate *priv)
 	guint32 serial_number = 0;
 	guint i, j;
 
-	/* get leds from HW */
-	ret = ch_device_cmd_get_leds (priv->device,
-				      &leds,
-				      &error);
+	/* get values from HW */
+	ch_device_queue_get_leds (priv->device_queue,
+				  priv->device,
+				  &leds);
+	ch_device_queue_get_color_select (priv->device_queue,
+					  priv->device,
+					  &color_select);
+	ch_device_queue_get_multiplier (priv->device_queue,
+					priv->device,
+					&multiplier);
+	ch_device_queue_get_firmware_ver (priv->device_queue,
+					  priv->device,
+					  &major,
+					  &minor,
+					  &micro);
+	ch_device_queue_get_serial_number (priv->device_queue,
+					   priv->device,
+					   &serial_number);
+	ch_device_queue_get_dark_offsets (priv->device_queue,
+					  priv->device,
+					  &value);
+	ch_device_queue_get_pre_scale (priv->device_queue,
+				       priv->device,
+				       &pre_scale);
+	ch_device_queue_get_post_scale (priv->device_queue,
+					priv->device,
+					&post_scale);
+	ch_device_queue_get_integral_time (priv->device_queue,
+					   priv->device,
+					   &integral_time);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
-		title = _("Failed to get LED status");
+		title = _("Failed to get device status");
 		ch_util_error_dialog (priv, title, error->message);
 		g_error_free (error);
 		goto out;
@@ -187,48 +194,12 @@ ch_util_refresh (ChUtilPrivate *priv)
 				      leds & 0x02);
 	priv->leds_old = leds;
 
-	/* get color select from HW */
-	ret = ch_device_cmd_get_color_select (priv->device,
-					      &color_select,
-					      &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get color select");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "comboboxtext_color_select"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), color_select);
 
-	/* get multiplier from HW */
-	ret = ch_device_cmd_get_multiplier (priv->device,
-					    &multiplier,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: the multiplier is the scale factor used
-		 * when using the sensor */
-		title = _("Failed to get multiplier");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "comboboxtext_multiplier"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), multiplier);
 
-	/* get firmware */
-	ret = ch_device_cmd_get_firmware_ver (priv->device,
-					      &major,
-					      &minor,
-					      &micro,
-					      &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get firmware version");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	tmp = g_strdup_printf ("%i", major);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_firmware_major"));
 	gtk_label_set_label (GTK_LABEL (widget), tmp);
@@ -242,15 +213,6 @@ ch_util_refresh (ChUtilPrivate *priv)
 	gtk_label_set_label (GTK_LABEL (widget), tmp);
 	g_free (tmp);
 
-	/* get firmware version */
-	ret = ch_device_cmd_get_serial_number (priv->device, &serial_number, &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get serial number");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	if (serial_number == 0xffffffff) {
 		g_warning ("no valid serial number");
 		serial_number = 0;
@@ -258,17 +220,6 @@ ch_util_refresh (ChUtilPrivate *priv)
 	adj = GTK_ADJUSTMENT (gtk_builder_get_object (priv->builder, "adjustment_serial"));
 	gtk_adjustment_set_value (adj, serial_number);
 
-	/* get dark offsets */
-	ret = ch_device_cmd_get_dark_offsets (priv->device,
-					      &value,
-					      &error);
-	if (!ret) {
-		/* TRANSLATORS: failed to get the absolute black offset */
-		title = _("Failed to get dark offsets");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	tmp = g_strdup_printf ("%.4f", value.R);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_dark_red"));
 	gtk_label_set_label (GTK_LABEL (widget), tmp);
@@ -283,12 +234,16 @@ ch_util_refresh (ChUtilPrivate *priv)
 	g_free (tmp);
 
 	/* get calibration */
-	ret = ch_device_cmd_get_calibration (priv->device,
-					     0,
-					     &calibration,
-					     NULL,
-					     NULL,
-					     &error);
+	ch_device_queue_get_calibration (priv->device_queue,
+					 priv->device,
+					 0,
+					 &calibration,
+					 NULL,
+					 NULL);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to get calibration data, resetting");
@@ -309,48 +264,18 @@ ch_util_refresh (ChUtilPrivate *priv)
 	}
 
 	/* get pre scale */
-	ret = ch_device_cmd_get_pre_scale (priv->device,
-					   &pre_scale,
-					   &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get pre scale");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	tmp = g_strdup_printf ("%.4f", pre_scale);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_pre_scale"));
 	gtk_label_set_label (GTK_LABEL (widget), tmp);
 	g_free (tmp);
 
 	/* get post scale */
-	ret = ch_device_cmd_get_post_scale (priv->device,
-					    &post_scale,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get post scale");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	tmp = g_strdup_printf ("%.4f", post_scale);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "label_post_scale"));
 	gtk_label_set_label (GTK_LABEL (widget), tmp);
 	g_free (tmp);
 
 	/* get integral time */
-	ret = ch_device_cmd_get_integral_time (priv->device,
-					       &integral_time,
-					       &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to get sample read time");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
 	switch (integral_time) {
 	case CH_INTEGRAL_TIME_VALUE_5MS:
 		i = 0;
@@ -407,9 +332,13 @@ ch_util_write_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	gboolean ret;
 	GError *error = NULL;
 
-	ret = ch_device_cmd_write_eeprom (priv->device,
-					  CH_WRITE_EEPROM_MAGIC,
-					  &error);
+	ch_device_queue_write_eeprom (priv->device_queue,
+				      priv->device,
+				      CH_WRITE_EEPROM_MAGIC);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to write EEPROM");
@@ -433,22 +362,17 @@ ch_util_measure_raw (ChUtilPrivate *priv)
 	GTimer *timer = NULL;
 
 	/* turn on sensor */
-	ret = ch_device_cmd_set_multiplier (priv->device,
-					    CH_FREQ_SCALE_100,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: we have to enable the sensor */
-		title = _("Failed to turn on sensor");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* get from HW */
 	timer = g_timer_new ();
-	ret = ch_device_cmd_take_readings (priv->device,
-					   &value,
-					   &error);
+	ch_device_queue_set_multiplier (priv->device_queue,
+					priv->device,
+					CH_FREQ_SCALE_100);
+	ch_device_queue_take_readings (priv->device_queue,
+				       priv->device,
+				       &value);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to take readings");
@@ -515,23 +439,18 @@ ch_util_measure_device (ChUtilPrivate *priv)
 	GTimer *timer = NULL;
 
 	/* turn on sensor */
-	ret = ch_device_cmd_set_multiplier (priv->device,
-					    CH_FREQ_SCALE_100,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to turn on sensor");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* get from HW */
 	timer = g_timer_new ();
-	ret = ch_device_cmd_take_readings_xyz (priv->device,
-					       0,
-					       &value,
-					       &error);
+	ch_device_queue_set_multiplier (priv->device_queue,
+					priv->device,
+					CH_FREQ_SCALE_100);
+	ch_device_queue_take_readings_xyz (priv->device_queue,
+					   priv->device,
+					   0,
+					   &value);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to take readings");
@@ -597,33 +516,19 @@ ch_util_dark_offset_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	value.R = 0.0f;
 	value.G = 0.0f;
 	value.B = 0.0f;
-	ret = ch_device_cmd_set_dark_offsets (priv->device,
-					      &value,
-					      &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to reset dark offsets");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* turn on sensor */
-	ret = ch_device_cmd_set_multiplier (priv->device,
-					    CH_FREQ_SCALE_100,
-					    &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to turn on sensor");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* get from HW */
-	ret = ch_device_cmd_take_readings (priv->device,
-					   &value,
-					   &error);
+	ch_device_queue_set_dark_offsets (priv->device_queue,
+					  priv->device,
+					  &value);
+	ch_device_queue_set_multiplier (priv->device_queue,
+					priv->device,
+					CH_FREQ_SCALE_100);
+	ch_device_queue_take_readings (priv->device_queue,
+				       priv->device,
+				       &value);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to take readings");
@@ -633,9 +538,13 @@ ch_util_dark_offset_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	}
 
 	/* set new values */
-	ret = ch_device_cmd_set_dark_offsets (priv->device,
-					      &value,
-					      &error);
+	ch_device_queue_set_dark_offsets (priv->device_queue,
+					  priv->device,
+					  &value);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set dark offsets");
@@ -660,110 +569,6 @@ ch_util_calibrate_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	ch_util_refresh (priv);
 }
 
-/**
- * ch_util_reset_button_cb:
- **/
-static void
-ch_util_reset_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
-{
-	const gchar *title;
-	gboolean ret;
-	GError *error = NULL;
-
-	/* set to HW */
-	ret = ch_device_cmd_reset (priv->device,
-				   &error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to reset processor");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* refresh */
-	ch_util_refresh (priv);
-out:
-	return;
-}
-
-/**
- * ch_util_get_firmware_filename:
- **/
-static gchar *
-ch_util_get_firmware_filename (GtkWindow *window)
-{
-	gchar *filename = NULL;
-	GtkWidget *dialog;
-	GtkFileFilter *filter;
-
-	/* TRANSLATORS: dialog for chosing the firmware */
-	dialog = gtk_file_chooser_dialog_new (_("Select firmware file"), window,
-					      GTK_FILE_CHOOSER_ACTION_OPEN,
-					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-					      NULL);
-	gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER(dialog), FALSE);
-
-	/* setup the filter */
-	filter = gtk_file_filter_new ();
-	/* TRANSLATORS: filter name on the file->open dialog */
-	gtk_file_filter_set_name (filter, _("Supported firmware files"));
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
-
-	/* setup the all files filter */
-	filter = gtk_file_filter_new ();
-	gtk_file_filter_add_pattern (filter, "*.bin");
-	/* TRANSLATORS: filter name on the file->open dialog */
-	gtk_file_filter_set_name (filter, _("Firmware images"));
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
-
-	/* did user choose file */
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog));
-
-	/* we're done */
-	gtk_widget_destroy (dialog);
-
-	/* or NULL for missing */
-	return filename;
-}
-
-/**
- * ch_util_flash_firmware_button_cb:
- **/
-static void
-ch_util_flash_firmware_button_cb (GtkWidget *widget, ChUtilPrivate *priv)
-{
-	const gchar *title;
-	gboolean ret;
-	GError *error = NULL;
-	GtkWindow *window;
-	gchar *filename;
-
-	window = GTK_WINDOW (gtk_builder_get_object (priv->builder, "dialog_ch"));
-	filename = ch_util_get_firmware_filename (window);
-	if (filename == NULL)
-		goto out;
-
-	/* set to HW */
-	ret = ch_client_flash_firmware (priv->client,
-					filename,
-					&error);
-	if (!ret) {
-		/* TRANSLATORS: internal device error */
-		title = _("Failed to flash new firmware");
-		ch_util_error_dialog (priv, title, error->message);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* refresh */
-	ch_util_refresh (priv);
-out:
-	g_free (filename);
-}
-
 /**
  * ch_util_color_select_changed_cb:
  **/
@@ -778,9 +583,13 @@ ch_util_color_select_changed_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	color_select = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 
 	/* set to HW */
-	ret = ch_device_cmd_set_color_select (priv->device,
-					      color_select,
-					      &error);
+	ch_device_queue_set_color_select (priv->device_queue,
+					  priv->device,
+					  color_select);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set color select");
@@ -801,7 +610,13 @@ ch_util_adjustment_value_changed_cb (GtkAdjustment *adjustment, ChUtilPrivate *p
 	GError *error = NULL;
 
 	value = gtk_adjustment_get_value (adjustment);
-	ret = ch_device_cmd_set_serial_number (priv->device, (guint64) value, &error);
+	ch_device_queue_set_serial_number (priv->device_queue,
+					   priv->device,
+					   (guint64) value);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set serial number");
@@ -823,12 +638,16 @@ ch_util_checkbutton0_toggled_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	priv->leds_old ^= CH_STATUS_LED_GREEN;
 
 	/* set to HW */
-	ret = ch_device_cmd_set_leds (priv->device,
-				      priv->leds_old,
-				      0,
-				      0xff,
-				      0xff,
-				      &error);
+	ch_device_queue_set_leds (priv->device_queue,
+				  priv->device,
+				  priv->leds_old,
+				  0,
+				  0xff,
+				  0xff);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set LEDs");
@@ -850,12 +669,16 @@ ch_util_checkbutton1_toggled_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	priv->leds_old ^= CH_STATUS_LED_RED;
 
 	/* set to HW */
-	ret = ch_device_cmd_set_leds (priv->device,
-				      priv->leds_old,
-				      0,
-				      0xff,
-				      0xff,
-				      &error);
+	ch_device_queue_set_leds (priv->device_queue,
+				  priv->device,
+				  priv->leds_old,
+				  0,
+				  0xff,
+				  0xff);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set LEDs");
@@ -878,9 +701,13 @@ ch_util_multiplier_changed_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	multiplier = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 
 	/* set to HW */
-	ret = ch_device_cmd_set_multiplier (priv->device,
-					    multiplier,
-					    &error);
+	ch_device_queue_set_multiplier (priv->device_queue,
+					priv->device,
+					multiplier);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set multiplier");
@@ -943,9 +770,13 @@ ch_util_integral_changed_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	}
 
 	/* set to HW */
-	ret = ch_device_cmd_set_integral_time (priv->device,
-					       integral_time,
-					       &error);
+	ch_device_queue_set_integral_time (priv->device_queue,
+					   priv->device,
+					   integral_time);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       &error);
 	if (!ret) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to set sample read time");
@@ -954,6 +785,41 @@ ch_util_integral_changed_cb (GtkWidget *widget, ChUtilPrivate *priv)
 	}
 }
 
+/**
+ * ch_util_get_default_device:
+ **/
+static GUsbDevice *
+ch_util_get_default_device (GError **error)
+{
+	gboolean ret;
+	GUsbContext *usb_ctx;
+	GUsbDevice *device = NULL;
+	GUsbDeviceList *list;
+
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	/* try to find the ColorHug device */
+	usb_ctx = g_usb_context_new (NULL);
+	list = g_usb_device_list_new (usb_ctx);
+	g_usb_device_list_coldplug (list);
+	device = g_usb_device_list_find_by_vid_pid (list,
+						    CH_USB_VID,
+						    CH_USB_PID,
+						    error);
+	if (device == NULL)
+		goto out;
+	g_debug ("Found ColorHug device %s",
+		 g_usb_device_get_platform_id (device));
+	ret = ch_device_open (device, error);
+	if (!ret)
+		goto out;
+out:
+	g_object_unref (usb_ctx);
+	if (list != NULL)
+		g_object_unref (list);
+	return device;
+}
+
 /**
  * ch_util_startup_cb:
  **/
@@ -990,7 +856,8 @@ ch_util_startup_cb (GApplication *application, ChUtilPrivate *priv)
 	gtk_widget_hide (main_window);
 
 	/* connect to device */
-	priv->device = ch_client_get_default (priv->client, &error);
+	priv->device_queue = ch_device_queue_new ();
+	priv->device = ch_util_get_default_device (&error);
 	if (priv->device == NULL) {
 		/* TRANSLATORS: internal device error */
 		title = _("Failed to connect to device");
@@ -1019,12 +886,6 @@ ch_util_startup_cb (GApplication *application, ChUtilPrivate *priv)
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_calibrate"));
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (ch_util_calibrate_button_cb), priv);
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_reset"));
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (ch_util_reset_button_cb), priv);
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_flash_firmware"));
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (ch_util_flash_firmware_button_cb), priv);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "comboboxtext_color_select"));
 	g_signal_connect (widget, "changed",
 			  G_CALLBACK (ch_util_color_select_changed_cb), priv);
@@ -1106,19 +967,16 @@ main (int argc, char **argv)
 	g_signal_connect (priv->application, "activate",
 			  G_CALLBACK (ch_util_activate_cb), priv);
 
-	/* use client */
-	priv->client = ch_client_new ();
-
 	/* wait */
 	status = g_application_run (G_APPLICATION (priv->application), argc, argv);
 
 	g_object_unref (priv->application);
 	if (priv->builder != NULL)
 		g_object_unref (priv->builder);
-	if (priv->client != NULL)
-		g_object_unref (priv->client);
 	if (priv->device != NULL)
 		g_object_unref (priv->device);
+	if (priv->device_queue != NULL)
+		g_object_unref (priv->device_queue);
 	g_free (priv);
 	return status;
 }
